# Solace PubSub+ Connector for TIBCO EMS
Solace
Corporation https://solace.com

*Revision: 2.0.0*
<br/>*Revision Date: 2023-11-30*

## Table of Contents

-   [Preface]
-   [Getting Started]
    -   [Prerequisites]
    -   [Usage]
    -   [Connecting to Services on the Host]
    -   [Configuring a Healthcheck]
-   [Providing Configuration]
-   [Ports]
-   [Volumes]
    -   [Volume: Spring Configuration Files]
    -   [Volume: Libraries]
    -   [Volume: Classpath Files]
    -   [Volume: Output Files]
-   [Configuring the JVM]
    -   [Support]
-   [License]

## Preface

Solace PubSub+ Connector for TIBCO EMS bridges data between the Solace PubSub+ Event Broker and TIBCO EMS providing a flexible and efficient way to integrate TIBCO EMS application data with your Solace-backed, event-driven architecture and the Event Mesh. The connector is deployable standalone or in redundancy modes of “active-standby” or “active-active” to allow for high-availability and horizontal scaling of your data movement. Each connector instance supports up to 20 individual workflows (source-to-target pipeline), minimizing the number of connector instances deployed and managed. The use of various Spring Framework technologies allows for easy configuration of the connector, advanced logging capabilities, and export of live metrics data to external monitoring solutions.

## Getting Started

[Release Notes]

### Prerequisites

-   [Docker] or [Podman]

-   [PubSub+ Event Broker]

-   [TIBCO EMS]

### Usage

1.  Create a directory called `libs`

2.  Download the jars for the TIBCO EMS Client and all its dependencies, if any, to the `libs` directory

3.  Create a directory called `config`

4.  Create an `application.yml` or `application.properties` file in the `config` directory containing the properties needed for your connector.

    -   For an example of such a config file, see the one packaged in the connector zip at <https://solace.com/connectors/pubsub-connector-for-tibco-ems>

5.  Run the container with minimal configuration:

    **Docker**

        docker run -d --name my-connector \
          -v `pwd`/libs/:/app/external/libs/:ro \
          -v `pwd`/config/:/app/external/spring/config/:ro \
          solace/solace-pubsub-connector-tibcoems:2.0.0

    **Podman**

        podman run -d --name my-connector \
          -v `pwd`/libs/:/app/external/libs/:ro \
          -v `pwd`/config/:/app/external/spring/config/:ro \
          solace/solace-pubsub-connector-tibcoems:2.0.0

### Connecting to Services on the Host

If services (e.g. PubSub+ event broker) are exposed on the localhost, they can be referred to using the container platform’s special DNS name which resolves to the internal IP address used by the host.

For example:

**Docker**

    docker run -d --name my-connector \
      -v `pwd`/libs/:/app/external/libs/:ro \
      -v `pwd`/config/:/app/external/spring/config/:ro \
      --env SOLACE_JAVA_HOST=host.docker.internal:55555 \
      solace/solace-pubsub-connector-tibcoems:2.0.0

**Podman**

    podman run -d --name my-connector \
      -v `pwd`/libs/:/app/external/libs/:ro \
      -v `pwd`/config/:/app/external/spring/config/:ro \
      --env SOLACE_JAVA_HOST=host.containers.internal:55555 \
      solace/solace-pubsub-connector-tibcoems:2.0.0

### Configuring a Healthcheck

Here’s a basic example command of how to configure the healthcheck for container:

**Docker**

    docker run -d --name my-connector \
      -v `pwd`/libs/:/app/external/libs/:ro \
      -v `pwd`/application.yml:/app/external/spring/config/application.yml:ro \
      --env SOLACE_CONNECTOR_SECURITY_USERS_0_NAME=healthcheck \
      --env SOLACE_CONNECTOR_SECURITY_USERS_0_PASSWORD=healthcheck \
      --healthcheck-command="curl -X GET -u healthcheck:healthcheck --fail localhost:8090/actuator/health" \
      solace/solace-pubsub-connector-tibcoems:2.0.0

**Podman**

    podman run -d --name my-connector \
      -v `pwd`/libs/:/app/external/libs/:ro \
      -v `pwd`/application.yml:/app/external/spring/config/application.yml:ro \
      --env SOLACE_CONNECTOR_SECURITY_USERS_0_NAME=healthcheck \
      --env SOLACE_CONNECTOR_SECURITY_USERS_0_PASSWORD=healthcheck \
      --healthcheck-command="curl -X GET -u healthcheck:healthcheck --fail localhost:8090/actuator/health" \
      solace/solace-pubsub-connector-tibcoems:2.0.0

This command does a few things:

-   Creates a regular read-only user called `healthcheck`.

-   Uses the `healthcheck` user as the user to poll the management health endpoint in the container’s `healthcheck-command` and fails it if the connector is unhealthy.

## Providing Configuration

There are two ways to provide Spring configuration properties to this container:

1.  Using [environment variables].

2.  Using [volume(s) containing Spring configuration files].

## Ports

<table>
<colgroup>
<col style="width: 50%" />
<col style="width: 50%" />
</colgroup>
<thead>
<tr class="header">
<th style="text-align: left;">Port</th>
<th style="text-align: left;">Usage</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td style="text-align: left;"><p><code>8090</code></p></td>
<td style="text-align: left;"><p>The connector’s management endpoint.</p></td>
</tr>
</tbody>
</table>

## Volumes

These are the supported directories for which volumes and bind mounts can be created.

<table>
<colgroup>
<col style="width: 20%" />
<col style="width: 40%" />
<col style="width: 20%" />
<col style="width: 20%" />
</colgroup>
<thead>
<tr class="header">
<th style="text-align: left;">Contents</th>
<th style="text-align: left;">Container Path</th>
<th style="text-align: left;">Optional</th>
<th style="text-align: left;">Recommended Permission</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td style="text-align: left;"><p>Spring configuration files</p></td>
<td style="text-align: left;"><p><code>/app/external/spring/config/</code></p></td>
<td style="text-align: left;"><p>Required unless all properties are defined using environment variables</p></td>
<td style="text-align: left;"><p>Read-Only</p></td>
</tr>
<tr class="even">
<td style="text-align: left;"><p>Libraries</p></td>
<td style="text-align: left;"><p><code>/app/external/libs/</code></p></td>
<td style="text-align: left;"><p>Required</p></td>
<td style="text-align: left;"><p>Read-Only</p></td>
</tr>
<tr class="odd">
<td style="text-align: left;"><p>Classpath files</p></td>
<td style="text-align: left;"><p><code>/app/external/classpath/</code></p></td>
<td style="text-align: left;"><p>Optional</p></td>
<td style="text-align: left;"><p>Read-Only</p></td>
</tr>
<tr class="even">
<td style="text-align: left;"><p>Output files</p></td>
<td style="text-align: left;"><p><code>/app/external/output/</code></p></td>
<td style="text-align: left;"><p>Optional</p></td>
<td style="text-align: left;"><p>Read/Write</p></td>
</tr>
</tbody>
</table>

### Volume: Spring Configuration Files

To add Spring configuration files (e.g. `application.yml`, etc), add a read-only volume or bind mount to `/app/external/spring/config/`.

This directory follows the same semantics as [Spring’s default `config/` directory]. That is to say, this connector will automatically find and load Spring configuration files from the following locations when the connector starts:

1.  The root of `/app/external/spring/config/`

2.  Immediate child directories of `/app/external/spring/config/`

> **TIP**
> 
> If config files for multiple, different, connectors will exist within the same config folder for use in different environments (e.g. dev, prod, etc), then consider using [Spring Boot Profiles] instead of child directories to do this.
> 
> i.e.:
> 
> -   Do this:
> 
>     -   `/app/external/spring/config/application-prod.yml`
> 
>     -   `/app/external/spring/config/application-dev.yml`
> 
> -   Instead of this:
> 
>     -   `/app/external/spring/config/prod/application.yml`
> 
>     -   `/app/external/spring/config/dev/application.yml`
> 
> Child directories are intended to be used for merging configuration from multiple sources of config properties. For more information and an example of when you might want to use multiple child directories to compose your application's configuration, please see the [Spring Boot documentation].
> 
>   [Spring Boot Profiles]: https://docs.spring.io/spring-boot/docs/current/reference/html/features.html#features.external-config.files.profile-specific
>   [Spring Boot documentation]: https://docs.spring.io/spring-boot/docs/current/reference/html/features.html#features.external-config.files.wildcard-locations

### Volume: Libraries

To add additional libraries, add a read-only volume or bind mount to `/app/external/libs/`.

This directory is provided as the location for the Java library dependencies (external `jar` files) that are
either required or
required only when using certain features of the Connector (such as Prometheus libraries when using the metrics export to Prometheus feature in your Connector configuration).

The required jars are not provided by Solace due to licensing considerations and need to be provided as part of the deployment of a Connector in order for the Connector to operate correctly.

See the documentation provided in the `libs` directory of the connector zip for more info.

### Volume: Classpath Files

To add arbitrary files (i.e. not jar libraries and not Spring Boot configuration files), add a read-only volume or bind mount to `/app/external/classpath/`.

> **NOTE**
> 
> This directory should not contain jar libraries nor Spring Boot configuration files. Otherwise, there is a risk of libraries not getting picked up by the connector and/or overwriting the connector's internal configuration.

### Volume: Output Files

Some features may support writing files (e.g. logging to a file). To capture these, add a read/write volume or bind mount to `/app/external/output/`.

> **IMPORTANT**
> 
> When using features which generates files, they must be configured so that files are generated to this directory. Generating files anywhere else is not supported.

## Configuring the JVM

To configure the JVM, set the `JDK_JAVA_OPTIONS` environment variable on the container.

See [the JDK documentation] for more info.

> **TIP**
> 
> This container has been tested using:
> 
> -   2 active processors → `-XX:ActiveProcessorCount=2`
> 
> -   2GB of max heap memory → `-Xmx2048m`

### Support

Support is offered best effort via our [Solace Developer Community].

Premium support options are available, please [Contact Solace].

## License

This project is licensed under the Solace Community License, Version 1.0. - See the `LICENSE` file under the container’s `/licenses` for details.

  [Preface]: #preface
  [Getting Started]: #getting-started
  [Prerequisites]: #prerequisites
  [Usage]: #usage
  [Connecting to Services on the Host]: #connecting-to-services-on-the-host
  [Configuring a Healthcheck]: #configuring-a-healthcheck
  [Providing Configuration]: #providing-configuration
  [Ports]: #ports
  [Volumes]: #volumes
  [Volume: Spring Configuration Files]: #_volume_spring_configuration_files
  [Volume: Libraries]: #volume-libraries
  [Volume: Classpath Files]: #volume-classpath-files
  [Volume: Output Files]: #volume-output-files
  [Configuring the JVM]: #configuring-the-jvm
  [Support]: #support
  [License]: #license
  [Release Notes]: https://products.solace.com/download/CONN_TIBCO_RN
  [Docker]: https://www.docker.com/
  [Podman]: https://podman.io/
  [PubSub+ Event Broker]: https://solace.com/products/event-broker/
  [TIBCO EMS]: https://www.tibco.com/products/tibco-enterprise-message-service
  [environment variables]: https://docs.spring.io/spring-boot/docs/current/reference/html/features.html#features.external-config.typesafe-configuration-properties.relaxed-binding.environment-variables
  [volume(s) containing Spring configuration files]: #_volume_spring_configuration_files
  [Spring’s default `config/` directory]: https://docs.spring.io/spring-boot/docs/current/reference/html/features.html#features.external-config.files
  [the JDK documentation]: https://docs.oracle.com/en/java/javase/17/docs/specs/man/java.html#using-the-jdk_java_options-launcher-environment-variable
  [Solace Developer Community]: https://solace.community/
  [Contact Solace]: https://solace.com/contact-us
